#!/usr/bin/ruby

module SwitchvoxFix

require 'uri'
require 'net/https'
require 'openssl'
require 'digest/md5'
require 'rubygems'
require 'json'

# Raised when credentials are incorrect
class LoginError < RuntimeError
end

# Raised when the response.body is empty
class EmptyResponse < RuntimeError
end

# Raised when a response is returned that we don't know how to handle
class UnhandledResponse < RuntimeError
end

# The primary class used to interact with Switchvox.
class Base 

  URL = "/json"
  attr :host, true
  attr :url, true
  attr :user, false
  attr :pass, false
  attr :connection, true
  attr :session, true
  attr :debug, true
  attr :auth_header, true

  def initialize(host, user, pass, port, options={})
    {:debug => false}.merge! options
    @debug = options[:debug]

    @host = host
    @user = user
    @pass = pass
    @url  = URI.parse("https://" + @host + URL)
    @url.port = port
   
    @ssl  = true 
        
    @connection  = false
    @auth_header = false
    login!
    raise LoginError, "Invalid Username or Password" unless logged_in?
  end

  # A standard REST call to get a list of entries
  def request(method, parameters={})
    login! unless logged_in?
    json = wrap_json(method, parameters)
    
    # Send the request
    header   = {'Content-Type' => "application/json"}
    request  = Net::HTTP::Post.new(@url.path, header)
    request.digest_auth(@user, @pass, @auth_header)
    request.body = json
    response = @connection.request(request)

    if @debug
      puts "#{method}: Request"
      puts json
      puts "\n"
    end

    case response
      when Net::HTTPOK
        raise EmptyResponse unless response.body
        response_json = JSON.parse response.body
        if @debug
          puts "#{method}: Response:"
          pp response_json 
          puts "\n\n"
        end
        response_obj = response_json["response"]["result"].to_obj
        return response_obj
      when Net::HTTPUnauthorized
        login!
        request(method, parameters)
      when Net::HTTPForbidden
        raise LoginError, "Invalid Username or Password" 
      else raise UnhandledResponse, "Can't handle response #{response}"
    end
  end

  protected

    # Check to see if we are logged in
    def logged_in?
      return false unless @auth_header
      true
    end
  
    # Attempt HTTP Digest Authentication with Switchvox
    def login!
      connect! unless connected?
      @auth_header = @connection.head(@url.path)
    end

    # Check to see if we have an HTTP/S Connection
    def connected?
      return false unless @connection
      return false unless @connection.started?
      true
    end

    # Connect to the remote Switchvox system.  If SSL is enabled, ignore certificate checking.
    def connect!
      @connection = Net::HTTP.new(@url.host, @url.port)
      if @ssl
        @connection.use_ssl = true
        @connection.verify_mode = OpenSSL::SSL::VERIFY_NONE
      end
      if @debug
        #@connection.set_debug_output $stderr
      end
      @connection.start
    end

    # Wrap a JSON method and parameters in a Switchvox compatible request body.
    def wrap_json(method, parameters={})
      json = <<-"EOF"
        {
          \"request\": {
            \"method\": \"#{method}\",
            \"parameters\": #{parameters.to_json}
          }
        }
      EOF
      json.gsub!(/^\s{8}/,'')
    end

end
  
end




